"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KafkaOrdererFactory = exports.KafkaOrderer = exports.KafkaOrdererConnection = void 0;
const protocol_definitions_1 = require("@fluidframework/protocol-definitions");
const core = __importStar(require("@fluidframework/server-services-core"));
/**
 * @internal
 */
class KafkaOrdererConnection {
    constructor(producer, tenantId, documentId, clientId, client, maxMessageSize, serviceConfiguration) {
        this.producer = producer;
        this.tenantId = tenantId;
        this.documentId = documentId;
        this.clientId = clientId;
        this.client = client;
        this.maxMessageSize = maxMessageSize;
        this.serviceConfiguration = serviceConfiguration;
    }
    static async create(producer, tenantId, documentId, client, maxMessageSize, clientId, serviceConfiguration) {
        // Create the connection
        return new KafkaOrdererConnection(producer, tenantId, documentId, clientId, client, maxMessageSize, serviceConfiguration);
    }
    /**
     * Sends the client join op for this connection
     */
    async connect(clientJoinMessageServerMetadata) {
        const clientDetail = {
            clientId: this.clientId,
            detail: this.client,
        };
        const operation = {
            clientSequenceNumber: -1,
            contents: null,
            data: JSON.stringify(clientDetail),
            referenceSequenceNumber: -1,
            traces: this.serviceConfiguration.enableTraces &&
                clientJoinMessageServerMetadata?.connectDocumentStartTime
                ? [
                    {
                        action: "ConnectDocumentStart",
                        service: "alfred",
                        timestamp: clientJoinMessageServerMetadata.connectDocumentStartTime,
                    },
                    {
                        action: "JoinRawOpStart",
                        service: "alfred",
                        timestamp: Date.now(),
                    },
                ]
                : undefined,
            type: protocol_definitions_1.MessageType.ClientJoin,
            serverMetadata: clientJoinMessageServerMetadata,
        };
        const message = {
            clientId: null,
            documentId: this.documentId,
            operation,
            tenantId: this.tenantId,
            timestamp: Date.now(),
            type: core.RawOperationType,
        };
        return this.submitRawOperation([message]);
    }
    /**
     * Orders the provided list of messages. The messages in the array are guaranteed to be ordered sequentially
     * so long as their total size fits under the maxMessageSize.
     */
    async order(messages) {
        const rawMessages = messages.map((message) => {
            const rawMessage = {
                clientId: this.clientId,
                documentId: this.documentId,
                operation: message,
                tenantId: this.tenantId,
                timestamp: Date.now(),
                type: core.RawOperationType,
            };
            return rawMessage;
        });
        return this.submitRawOperation(rawMessages);
    }
    /**
     * Sends the client leave op for this connection
     */
    async disconnect(clientLeaveMessageServerMetadata) {
        const operation = {
            clientSequenceNumber: -1,
            contents: null,
            data: JSON.stringify(this.clientId),
            referenceSequenceNumber: -1,
            traces: this.serviceConfiguration.enableTraces ? [] : undefined,
            type: protocol_definitions_1.MessageType.ClientLeave,
            serverMetadata: clientLeaveMessageServerMetadata,
        };
        const message = {
            clientId: null,
            documentId: this.documentId,
            operation,
            tenantId: this.tenantId,
            timestamp: Date.now(),
            type: core.RawOperationType,
        };
        return this.submitRawOperation([message]);
    }
    once(event, listener) {
        this.producer.once(event, listener);
    }
    async submitRawOperation(messages) {
        if (this.serviceConfiguration.enableTraces) {
            // Add trace
            messages.forEach((message) => {
                const operation = message.operation;
                if (operation && operation.traces === undefined) {
                    operation.traces = [];
                }
                else if (operation?.traces && operation.traces.length > 0) {
                    operation.traces.push({
                        action: "end",
                        service: "alfred",
                        timestamp: Date.now(),
                    });
                }
            });
        }
        return this.producer.send(messages, this.tenantId, this.documentId);
    }
}
exports.KafkaOrdererConnection = KafkaOrdererConnection;
/**
 * @internal
 */
class KafkaOrderer {
    constructor(producer, tenantId, documentId, maxMessageSize, serviceConfiguration) {
        this.producer = producer;
        this.tenantId = tenantId;
        this.documentId = documentId;
        this.maxMessageSize = maxMessageSize;
        this.serviceConfiguration = serviceConfiguration;
    }
    static async create(producer, tenantId, documentId, maxMessageSize, serviceConfiguration) {
        return new KafkaOrderer(producer, tenantId, documentId, maxMessageSize, serviceConfiguration);
    }
    async connect(socket, clientId, client) {
        const connection = KafkaOrdererConnection.create(this.producer, this.tenantId, this.documentId, client, this.maxMessageSize, clientId, this.serviceConfiguration);
        return connection;
    }
    // eslint-disable-next-line @typescript-eslint/promise-function-async
    close() {
        return Promise.resolve();
    }
}
exports.KafkaOrderer = KafkaOrderer;
/**
 * @internal
 */
class KafkaOrdererFactory {
    constructor(producer, maxMessageSize, serviceConfiguration) {
        this.producer = producer;
        this.maxMessageSize = maxMessageSize;
        this.serviceConfiguration = serviceConfiguration;
        this.ordererMap = new Map();
    }
    async create(tenantId, documentId) {
        const fullId = `${tenantId}/${documentId}`;
        let orderer = this.ordererMap.get(fullId);
        if (orderer === undefined) {
            orderer = KafkaOrderer.create(this.producer, tenantId, documentId, this.maxMessageSize, this.serviceConfiguration);
            this.ordererMap.set(fullId, orderer);
        }
        return orderer;
    }
    delete(tenantId, documentId) {
        this.ordererMap.delete(`${tenantId}/${documentId}`);
    }
}
exports.KafkaOrdererFactory = KafkaOrdererFactory;
//# sourceMappingURL=kafkaOrderer.js.map